/*
 * ToolLabelsFinder for Tool Labels Plugin for Jenkins
 * Copyright (C) 2014 Kaz Nishimura
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Affero General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.vx68k.hudson.plugin.tool.labels;

import java.util.Collection;
import java.util.HashSet;
import java.util.StringTokenizer;
import hudson.Extension;
import hudson.model.Descriptor;
import hudson.model.Hudson;
import hudson.model.LabelFinder;
import hudson.model.Node;
import hudson.model.labels.LabelAtom;
import hudson.tools.ToolDescriptor;
import hudson.tools.ToolInstallation;
import hudson.tools.ToolLocationNodeProperty;
import hudson.tools.ToolProperty;

/**
 * Adds labels dynamically to a node with any tool installations.
 *
 * @author Kaz Nishimura
 * @since 2.0
 */
@Extension
public class ToolLabelsFinder extends LabelFinder {
    private Object ToolInstallataion;

    public ToolLabelsFinder() {
    }

    protected ToolLabelsProperty getToolLabelsProperty(
            ToolInstallation installation) {
        for (ToolProperty<?> property : installation.getProperties()) {
            if (property instanceof ToolLabelsProperty) {
                return (ToolLabelsProperty) property;
            }
        }
        return null;
    }

    protected void addLabels(Collection<LabelAtom> labels,
            ToolLocationNodeProperty tools, ToolDescriptor<?> descriptor) {
        Hudson application = Hudson.getInstance();
        for (ToolInstallation installation : descriptor.getInstallations()) {
            ToolLabelsProperty property = getToolLabelsProperty(installation);
            if (property != null) {
                String home = null;
                if (tools != null) {
                    home = tools.getHome(installation);
                }
                if (home == null) {
                    home = installation.getHome();
                }
                if (!home.isEmpty()) {
                    StringTokenizer tokenizer =
                            new StringTokenizer(property.getLabels());
                    while (tokenizer.hasMoreTokens()) {
                        String token = tokenizer.nextToken();
                        labels.add(application.getLabelAtom(token));
                    }
                }
            }
        }
    }

    @Override
    public Collection<LabelAtom> findLabels(Node node) {
        Hudson application = Hudson.getInstance();
        ToolLocationNodeProperty tools =
                node.getNodeProperties().get(ToolLocationNodeProperty.class);

        HashSet<LabelAtom> labels = new HashSet();
        for (Descriptor<ToolInstallation> descriptor :
                application.getDescriptorList(ToolInstallation.class)) {
            assert descriptor instanceof ToolDescriptor<?>;
            addLabels(labels, tools, (ToolDescriptor<?>) descriptor);
        }
        return labels;
    }
}
